#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define MAX_WORD_LEN 20

/* TypeDefs */
struct linkList {
   char word[MAX_WORD_LEN];
   char suffix[MAX_WORD_LEN];
   struct linkList *next;
};

struct statistics {
	int numberOfEqualAnalyzedWords;
	int numberOfDifferentAnalyzedWords;
	int numberOfWordsInOneAnalyzeOnly1;
	int numberOfWordsInOneAnalyzeOnly2;
};

/* Global Variables */
FILE     *autoMorphFile;
FILE     *linguisticaFile;
struct linkList *autoMorphLLHeader = NULL;
struct linkList *linguisticaLLHeader = NULL;
struct statistics	stat;

/* Internal Functions */
int openFiles (int argc, char *argv[]);
int closeFiles (void);
int linkListAdd (struct linkList** listHeader, char newWord[MAX_WORD_LEN], char newSuffix[MAX_WORD_LEN]);
int linkListSearchandDelete (struct linkList** listHeader, char Word[MAX_WORD_LEN],  struct linkList* returnedElement);
int linkListSearch (struct linkList** listHeader, char Word[MAX_WORD_LEN], struct linkList* returnedElement );
int linkListCount(struct linkList** listHeader);
int linkListDelete (struct linkList** listHeader, char Word[MAX_WORD_LEN]);
int linkListPrint(struct linkList** listHeader);
int readFile(FILE* workingFile, struct linkList** listHeader);
int analyze(struct linkList** listHeader1, struct linkList** listHeader2);

/* main funaction */
int main(int argc, char *argv[])
{

   int retStat;
	stat.numberOfDifferentAnalyzedWords = 0;
	stat.numberOfEqualAnalyzedWords = 0;
	stat.	numberOfWordsInOneAnalyzeOnly1 = 0;
	stat.	numberOfWordsInOneAnalyzeOnly2 = 0;

   printf ("In main\n");

   retStat = openFiles(argc, argv);
   if (retStat)
   {
      printf ("Error in function openFiles, exit program\n");  
      return 1;
   }

	retStat = readFile(autoMorphFile, &autoMorphLLHeader);
   if (retStat)
   {
      closeFiles();
      return 0;
   }
	retStat = readFile(linguisticaFile, &linguisticaLLHeader);
   if (retStat)
   {
      closeFiles();
      return 0;
   }
	closeFiles();

	retStat = linkListPrint(&autoMorphLLHeader);
   if (retStat)
   {
      return 0;
   }
	retStat = linkListPrint(&linguisticaLLHeader);
   if (retStat)
   {
      return 0;
   }

	analyze(&autoMorphLLHeader, &linguisticaLLHeader);
   return 0;

}

int analyze(struct linkList** listHeader1, struct linkList** listHeader2)
{
	struct linkList* firstList = *listHeader1;
	struct linkList  tempElement;
	int	 retStat;

	firstList = firstList;
	do
	{

		retStat = linkListSearchandDelete(listHeader2, firstList->word, &tempElement);
		if (retStat)
		{
			printf ("Analyze - the word %s, doesn't appear in the second list\n", firstList->word);
			stat.numberOfWordsInOneAnalyzeOnly1++;
		}
		else
		{
			if(!strcmp(firstList->suffix,tempElement.suffix))
			{
				printf ("Analyze - the word %s, has the same suffix in both lists (%s)\n", firstList->word, firstList->suffix);
				stat.numberOfEqualAnalyzedWords++;
			}
			else
			{
				printf ("Analyze - the word %s, has different suffixes(%s, %s)\n", firstList->word, firstList->suffix, tempElement.suffix);
				stat.numberOfDifferentAnalyzedWords++;
			}
		}

		firstList = firstList->next;
	} while (firstList);

	/* count the rest of the word which appears in one list only */
	stat.numberOfWordsInOneAnalyzeOnly2 = linkListCount(listHeader2);

	printf ("\nResults:\n");
	printf ("========\n");
	printf ("Same analyze              =  %d\n", stat.numberOfEqualAnalyzedWords);
	printf ("Different analyze         =  %d\n", stat.numberOfDifferentAnalyzedWords);
	printf ("Words in one list only(1) =  %d\n", stat.numberOfWordsInOneAnalyzeOnly1);
	printf ("Words in one list only(2) =  %d\n", stat.numberOfWordsInOneAnalyzeOnly2);
	return 0;

}

int linkListPrint(struct linkList** listHeader)
{
   struct linkList *tempElement;

   if (*listHeader == NULL)
   {
		printf ("linkListPrint - No Element in the list\n");
		return 1;
   }

	printf ("Words in the list:");

   /* Search in the link list for the proper place to insert the new element */
   tempElement = *listHeader;
   do
   {
		printf (" %s", tempElement->word);
		tempElement = tempElement->next;
   } while (tempElement);
	
	printf ("\n");
	return 0;
}

int linkListCount(struct linkList** listHeader)
{
   struct linkList *tempElement;
   int tempCount = 0;

   if (*listHeader == NULL)
   {
		printf ("linkListCount - No Element in the list\n");
      return tempCount;
   }

	printf ("Word left in list:");

   /* Search in the link list for the proper place to insert the new element */
   tempElement = *listHeader;
   do
   {
		tempCount++;
		printf (" %s", tempElement->word);
		tempElement = tempElement->next;
   } while (tempElement);
	
	printf ("\n");
	return tempCount;
}

int readFile(FILE* workingFile, struct linkList** listHeader)
{
	int x;
	char	newStem[MAX_WORD_LEN], newWord[MAX_WORD_LEN], newSuffix[MAX_WORD_LEN];
	int 	state = 0, tempLen;

   printf ("In readFile\n");

	newStem[0] = '\0';
	newSuffix[0]  = '\0';
	while (!feof(workingFile))
	{
		x = fgetc(workingFile);
		if (x=='\t')
		{
			state = 1;
			continue;
		}
		else if (x=='.')
		{
			if (!strcmp(newSuffix,"NULL"))
			{
				strncpy (newWord, newStem, strlen(newStem));
				newWord[strlen(newStem)] = '\0';
			}
			else
			{
				if (strlen(newStem) + strlen(newSuffix) > MAX_WORD_LEN - 1)
				{
					printf ("Error, stem + suffix too long - %s, %s (max %d letters)\n", newStem, newSuffix, MAX_WORD_LEN);
					return 1;
				}
				strncpy (newWord, newStem, strlen(newStem));
				newWord[strlen(newStem)] = '\0';
				strncat (newWord, newSuffix, strlen(newSuffix));
				newWord[strlen(newStem)+strlen(newSuffix)] = '\0';
			}
			if (strlen(newWord))
			{
				linkListAdd(listHeader, newWord, newSuffix);
			}
			newSuffix[0] = '\0';
			newWord[0] = '\0';
			continue;
		}
		else if (x=='\n')
		{
			if (!strcmp(newSuffix,"NULL"))
			{
				strncpy (newWord, newStem, strlen(newStem));
				newWord[strlen(newStem)] = '\0';
			}
			else
			{
				if (strlen(newStem) + strlen(newSuffix) > MAX_WORD_LEN - 1)
				{
					printf ("Error, stem + suffix too long - %s, %s (max %d letters)\n", newStem, newSuffix, MAX_WORD_LEN);
					return 1;
				}
				strncpy (newWord, newStem, strlen(newStem));
				newWord[strlen(newStem)] = '\0';
				strncat (newWord, newSuffix, strlen(newSuffix));
				newWord[strlen(newStem)+strlen(newSuffix)] = '\0';
			}
			if (strlen(newWord))
			{
				linkListAdd(listHeader, newWord, newSuffix);
			}
			newWord[0] = '\0';
			newStem[0] = '\0';
			newSuffix[0] = '\0';
			state = 0;
			continue;
		}

		switch (state)
		{
			case 0:	tempLen = strlen(newStem);
						if (tempLen > MAX_WORD_LEN - 2)
						{
								printf ("Error, stem too long - %s (max %d letters)\n", newStem, MAX_WORD_LEN);
								return 1;
						}
						newStem[tempLen] = x;
						newStem[tempLen+1] = '\0';
						break;
			case 1:	tempLen = strlen(newSuffix);
						if (tempLen > MAX_WORD_LEN - 2)
						{
								printf ("Error, suffix too long - %s (max %d letters)\n", newSuffix, MAX_WORD_LEN);
								return 1;
						}
						newSuffix[tempLen] = x;
						newSuffix[tempLen+1] = '\0';
						break;
			default: printf ("Error: Illegal state in the state machine\n");
						break;

		}

	}

	return 0;
}

int closeFiles (void)
{
	 printf ("In closeFiles\n");
    fclose(autoMorphFile);
    fclose(linguisticaFile);
	 return 0;

}

int openFiles (int argc, char *argv[])
{
    int x;

	 printf ("In openFiles\n");

    if (argc != 3)
    {
      printf ("--Error: The lingcomp should get 2 input files:\n");
      printf (" syntax: lingcomp autoMorphFile linguisticaFile\n");
      return 1;
    }

    autoMorphFile=fopen(argv[1],"r");
    if (!autoMorphFile)
    {
        printf ("Can't open file %s\n", argv[1]);
        return 1;
    }

    linguisticaFile=fopen(argv[2],"r");
    if (!linguisticaFile)
    {
        printf ("Can't open file %s\n", argv[2]);
        return 1;
    }

    return 0;   

}

int linkListAdd (struct linkList** listHeader, char newWord[MAX_WORD_LEN], char newSuffix[MAX_WORD_LEN])
{
   struct linkList *newLinkListElement;
   struct linkList *tempElement, *prevElement;
   int cmpResult;


   /* prepare new linl list element */
   newLinkListElement = (struct linkList*) malloc (sizeof(struct linkList));
   if (newLinkListElement == NULL)
   {
      printf ("Error allocating mameory for new linkList element");
      return 1;
   }
   strcpy(newLinkListElement->word, newWord);
   strcpy(newLinkListElement->suffix, newSuffix);

   newLinkListElement->next = NULL;
   
   /* If this is the first time, the header will be the new element */   
   if (*listHeader == NULL)
   {
      *listHeader = newLinkListElement;
		//printf ("First time in linkListAdd, initializing the header\n");
      return 0;
   }

   /* Search in the link list for the proper place to insert the new element */
   tempElement = *listHeader;
   prevElement = NULL;
   do
   {
      cmpResult = strcmp (newLinkListElement->word, tempElement->word);
      if (cmpResult == 0)
      {
         printf ("Duplicate word in the list: %s (suffix = %s)\n", tempElement->word, tempElement->suffix);
         return 0;
      }
      if (cmpResult < 0)
      {
			if (prevElement == NULL)
			{
				*listHeader = newLinkListElement;
				newLinkListElement->next = tempElement;	
				//printf ("Put %s, in the head of the list, and before %s\n", newLinkListElement->word, tempElement->word);
				return 0;
			}
			else
			{
				newLinkListElement->next = tempElement;
				prevElement->next = newLinkListElement;
				//printf ("Put %s, after %s, and before %s\n", newLinkListElement->word, prevElement->word, tempElement->word);
				return 0;
			}
      }

      prevElement = tempElement;
		tempElement = tempElement->next;
   } while (tempElement);

	/* The new element is in the end of the list */
	prevElement->next = newLinkListElement;
	//printf ("Put %s, in the tail of the list, and after %s\n", newLinkListElement->word, prevElement->word);
	
	return 0;
}


int linkListSearch (struct linkList** listHeader, char Word[MAX_WORD_LEN], struct linkList* returnedElement )
{
   struct linkList *newLinkListElement;
   struct linkList *tempElement, *prevElement;
   int cmpResult;


   /* If this is the first time, the header will be the new element */   
   if (*listHeader == NULL)
   {
		printf ("linkListSearch - No Element in the list\n");
      return 1;
   }

   /* Search in the link list for the proper place to insert the new element */
   tempElement = *listHeader;
   do
   {
      cmpResult = strcmp (tempElement->word, Word);
      if (cmpResult == 0)
      {
         //printf ("Found %s in the list\n", Word);
			memcpy (returnedElement, tempElement, sizeof(struct linkList));
         return 0;
      }
		else if (cmpResult > 0)
		{
			//printf ("Didn't find %s in the list, current word = %s\n", Word, tempElement->word);
			return 1;
		}

		tempElement = tempElement->next;
   } while (tempElement);

	/* The new element is in the end of the list */
	//printf ("Didn't find %s in the list\n", Word);
	return 1;
}

int linkListSearchandDelete (struct linkList** listHeader, char Word[MAX_WORD_LEN],  struct linkList* returnedElement)
{
   struct linkList *newLinkListElement;
   struct linkList *tempElement, *prevElement;
   int cmpResult;

   /* If this is the first time, the header will be the new element */   
   if (*listHeader == NULL)
   {
		printf ("linkListSearch - No Element in the list\n");
      return 1;
   }

   /* Search in the link list for the word */
	prevElement = NULL;
   tempElement = *listHeader;
   do
   {
      cmpResult = strcmp (tempElement->word, Word);
      if (cmpResult == 0)
      {
         //printf ("Deleting %s from the list\n", Word);
			memcpy (returnedElement, tempElement, sizeof(struct linkList));
			if (prevElement==NULL)
			{
				tempElement = *listHeader;
				*listHeader = (*listHeader)->next;
			}
			else
			{
				prevElement->next = tempElement->next;
			}
			free(tempElement);
         return 0;
      }
		else if (cmpResult > 0)
		{
			//printf ("Didn't find %s in the list, current word = %s\n", Word, tempElement->word);
			return 1;
		}

		prevElement = tempElement;
		tempElement = tempElement->next;
   } while (tempElement);

	/* The new element is in the end of the list */
	//printf ("Didn't find %s in the list\n", Word);
	return 1;
}
