package automorphology.core;

import java.io.*;

/**
 * <p>Title: Auto-Morphology</p>
 * <p>Description: A morphological analyzer based on Goldsmith's algorithm</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Haifa University</p>
 * @author Yaniv Alamaru, Ofer Senderovich
 * @version 1.0
 */

public class CLParser {
      static private boolean prefix = false;
      public static void main(String[] args) {
            int index = 0;
            if (!(parseInput(args))) {
                  usage(System.err);
                  System.exit(1);
            }
            long systemRunTime = System.currentTimeMillis()/1000;
            Corpus corpus = new Corpus(args[0],args[1]);
            index+=2;
            try {
                  System.err.println("Reading Corpus File.");
                  long numberOfWords = corpus.init(null,prefix);
                  System.err.println("Corpus contains " + numberOfWords + " tokens.");
                  System.err.println("Corpus contains " + corpus.getWords().size() + " words");
            }
            catch(FileNotFoundException exc) {
                  exc.printStackTrace();
            }
            IBootstrapHeuristic bootstrap = null;
            if (prefix)
                  index++;
            switch(Integer.parseInt(args[index++])) {
                  case 1: /* Kgram Heuristic */
                        bootstrap = new KgramHeuristic();
                        System.err.println("Performing Kgram Heuristic.");
                        break;
                  case 2:
                        bootstrap = new SuccessorPeakHeuristic();
                        System.err.println("Performing Successor Peak Heuristic.");
                        break;
                  default:
                        System.err.println("Unknown bootstrap heuristic code.");
                        System.exit(1);
            }
            IMorphology morpholohy = bootstrap.analyze(corpus,null);
            double morphology_mdl = -1;
            for(int i=index; i<args.length;i++) {
                  IIncrementalHeuristic incrementalHeuristic = null;
                  switch (Integer.parseInt(args[i])) {
                         case 1: /* Cleanup Heuristic */
                               incrementalHeuristic = new CleanupHeuristic(2,2);
                               System.err.println("Performing Cleanup Heuristic.");
                               break;
                        case 2: /* Suffix Reducer Heuristic */
                              System.err.println("Performing Suffix Reducer Heuristic.");
                              incrementalHeuristic = new SuffixReducerHeuristic();
                              break;
                        default:
                              System.err.println("Unknown inceremental heuristic.");
                              System.exit(1);
                  }
                  IMorphology morpholohy2 = incrementalHeuristic.analyze(corpus,morpholohy);
                  IMDL mdl = new MDL();
                  System.err.println("Caculating the description length.");
                  double morphology2_mdl = mdl.calc(corpus,morpholohy2);
                  if (morphology_mdl == -1) {
                        System.err.println("Caculating the description length.");
                        morphology_mdl = mdl.calc(corpus,morpholohy);
                  }
                  if (morphology_mdl > morphology2_mdl) {
                        System.err.println("Incremental Heuristic produced an imporved morphology by " + (morphology_mdl - morphology2_mdl) +  " bits."  );
                        morpholohy = morpholohy2;
                        morphology_mdl = morphology2_mdl;
                  }
                  else {
                        System.err.println("Incremental Heuristic produced less imporved morphology by " + (morphology2_mdl - morphology_mdl) +  " bits."  );
                  }
            }
            morpholohy.printAnalysis(System.out,prefix);
            System.err.println("System ran for " + (System.currentTimeMillis()/1000 - systemRunTime) + " seconds.");
      }

      public static void usage(PrintStream err) {
            err.println("Usage: java.exe AutoMorphology.core.CLParser <corpus file> <alphabeta file> [-prefix] <bootstrap heuristic> <incremental heuristic>*");
            err.println("Bootstrap heuristics");
            err.println("1) Kgram Heuristic");
            err.println("2) Successor Peak Heuristic");
            err.println("Incremental Heuristic");
            err.println("1) Cleanup Heuristic");
            err.println("2) Suffix Reducer Heuristic");
      }

      public static boolean  parseInput(String[] args) {
            int index=0;
            if (args.length < 3) {
                  System.err.println("Invalid number of arguments");
                  return false;
            }

            String corpusFilename = args[index++];
            String alphaFilename = args[index++];
            int bootstrapHeuristic;
            try {
                  bootstrapHeuristic = Integer.parseInt(args[index++]);
            }
            catch (NumberFormatException ex) {
                  index--;
                  String tmp = new String(args[index++]);
                  if (tmp.compareTo("-prefix") == 0) {
                        prefix = true;
                  }
                  else {
                        System.err.println("Illegal parameter " + tmp);
                        return false;
                  }
                  bootstrapHeuristic = Integer.parseInt(args[index++]);
            }
            if (bootstrapHeuristic<1 || bootstrapHeuristic>2) {
                  System.err.println("Invalid bootstrap heurisitic " + bootstrapHeuristic);
                  return false;
            }
            int[] incrementalHeuristics = new int[10];
            for(int i=index; i<args.length;i++) {
                  incrementalHeuristics[i-3] = Integer.parseInt(args[i]);
                  if (incrementalHeuristics[i-3]<1 || incrementalHeuristics[i-3]>2) {
                        System.err.println("Invalid incremental heurisitic " + bootstrapHeuristic);
                        return false;
                  }
            }
            File corpusFile = new File(corpusFilename);
            if (!(corpusFile.exists())) {
                  System.err.println("corpus file not found");
                  return false;
            }
            File alphaFile = new File(alphaFilename);
            if (!(alphaFile.exists())) {
                  System.err.println("alphabeta file not found");
                  return false;
            }
            return true;
      }
}