package automorphology.core;

import java.util.*;

/**
 * <p>Title: Auto-Morphology</p>
 * <p>Description: A morphological analyzer based on Goldsmith's algorithm</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Haifa University</p>
 * @author Yaniv Alamaru, Ofer Senderovich
 * @version 1.0
 */

public class CleanupHeuristic implements IIncrementalHeuristic {

      private int minStemSize;
      private int minSuffixSize;

      public CleanupHeuristic(int minStemSize,int minSuffixSize) {
            this.minStemSize = minStemSize;
            this.minSuffixSize = minSuffixSize;
      }

      public IMorphology analyze(ICorpus corpus, IMorphology oldMorphology) {
            int progressMark = 0;
            IMorphology newMorphology = oldMorphology.getNewMorphology();
            Iterator signIter = oldMorphology.getSignatures().iterator();
            while(signIter.hasNext()) {
                  ISignature signature = (ISignature)signIter.next();
                  if (((++progressMark % 100) == 0) || (signature.getStems().size()>200)) {
                        System.err.print('.');
                        System.err.flush();
                  }
                  if (signature.getStems().size() <= minStemSize && signature.getSuffixs().size() <= minSuffixSize) {
                        Iterator wordIter = signature.getWords().iterator();
                        while(wordIter.hasNext()) {
                              String word = (String)wordIter.next();
                              newMorphology.getNullSigature().addStem(new SimpleStem(word));
                        }
                        updateFrequencies(corpus,newMorphology,signature);
                        continue;
                  }
                  newMorphology.addSignature(signature.copy());
            }
            System.err.println();
            System.err.flush();
            return newMorphology;
      }

      private void updateFrequencies(ICorpus corpus,IMorphology morphology,ISignature signature) {
            Iterator suffixIter = signature.getSuffixs().iterator();
            ISuffix nullSuffix = new Suffix("#");
            while(suffixIter.hasNext()) {
                  ISuffix suffix = (ISuffix)suffixIter.next();
                  morphology.setSuffixFrequency(suffix,morphology.getSuffixFrequency(suffix) - corpus.getSuffixInSignatureFrequency(signature,suffix));
                  morphology.setSuffixFrequency(nullSuffix,morphology.getSuffixFrequency(nullSuffix) + corpus.getSuffixInSignatureFrequency(signature,suffix));
            }
            Iterator stemIter = signature.getStems().iterator();
            while(stemIter.hasNext()) {
                  IStem stem = (IStem)stemIter.next();
                  morphology.setStemFrequency(stem,0);
                  suffixIter = signature.getSuffixs().iterator();
                  while(suffixIter.hasNext()) {
                        ISuffix suffix = (ISuffix)suffixIter.next();
                        String word;
                        if (suffix.toString().compareTo("#") == 0)
                              word = new String(stem.toString());
                        else
                              word = new String(stem.toString() + suffix.toString());
                        morphology.setStemFrequency(new SimpleStem(word),corpus.getWordFrequency(word));
                  }
            }
      }
}