package automorphology.core;

import java.io.*;
import java.util.*;
import javax.swing.*;

/**
 * <p>Title: Auto-Morphology</p>
 * <p>Description: A morphological analyzer based on Goldsmith's algorithm</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Haifa University</p>
 * @author Yaniv Alamaru, Ofer Senderovich
 * @version 1.0
 */

public class Corpus implements ICorpus {

      private File corpusFile;
      private File corpusAlphaBet;
      private HashMap wordAppearances;
      private HashMap letterAppearances;
      private HashMap suffixAppearances;
      private HashMap stemAppearances;
      private HashMap totalKgrams;
      private int numberOfWords;

      private boolean isLetterInString(String str,char letter) {
            for(int i=0;i<str.length();i++) {
                  if (letter == str.charAt(i))
                        return true;
            }
            return false;
      }

      private String readWord(Reader reader,String alphaBet,JProgressBar progressBar,boolean prefix) {
            StringBuffer word= new StringBuffer();
            Character letter;
            int read_result;
            if (progressBar == null)
                  progressBar = new JProgressBar();
            /* Skip leading garbage */
            do {
                  try {
                        read_result = reader.read();
                        progressBar.setValue(progressBar.getValue()+1);
                  }
                  catch(IOException exception) {
                        return new String("");
                  }
                  if (read_result == -1)
                        return new String("");
            } while (!isLetterInString(alphaBet,(char)read_result));
            while (isLetterInString(alphaBet,(char)read_result)) {
                  Character ch = new Character(Character.toLowerCase((char)read_result));
                  word.append(ch.charValue());
                  Integer numberOfAppearances = (Integer)letterAppearances.get((Object)ch);
                  if (numberOfAppearances == null)
                        numberOfAppearances = new Integer(1);
                  else
                        numberOfAppearances = new Integer((int)numberOfAppearances.intValue() + 1);
                  letterAppearances.put(ch,numberOfAppearances);
                  try {
                        read_result = reader.read();
                        reader.mark(2);
                        if ((char)read_result == '-') {
                              read_result = reader.read();
                              if ((char)read_result != '\r') {
                                    reader.reset();
                                    progressBar.setValue(progressBar.getValue()+1);
                                    break;
                              }
                              else {
                                    read_result = reader.read();
                                    read_result = reader.read();
                                    progressBar.setValue(progressBar.getValue()+3);
                              }
                        }
                        progressBar.setValue(progressBar.getValue()+1);
                  }
                  catch(IOException exception) {
                        if (prefix)
                              return word.reverse().toString();
                        else
                              return word.toString();
                  }
            }
            if (prefix)
                  return word.reverse().toString();
            else
                  return word.toString();
      }

      public Corpus(String corpusFile,String corpusAlhpaBet) {
            this.corpusFile = new File(corpusFile);
            this.corpusAlphaBet = new File(corpusAlhpaBet);
            wordAppearances = new HashMap();
            letterAppearances = new HashMap();
            suffixAppearances = new HashMap();
            stemAppearances = new HashMap();
            totalKgrams = new HashMap();
            numberOfWords = 0;
      }

      private String getAlphaBet() throws FileNotFoundException {
            Reader reader = new BufferedReader(new FileReader(corpusAlphaBet));
            StringBuffer alphaBet = new StringBuffer();
            try {
                  int letter = reader.read();
                  while(letter != -1) {
                        alphaBet.append((char)letter);
                        letter = reader.read();
                  }
            }
            catch(IOException exception) {
                  exception.printStackTrace();
            }
            return alphaBet.toString();
      }

      /**
       * The method initializes the internal state of the corpus by loading
       * the corpus file and populating the inner data structures. The method
       * must be called before any other method in the class.
       * @param progressBar To inform the client about the progress of the initialization.
       * @throws FileNotFoundException In case the corpus file is not found this exception is thrown.
       */
      public int init(JProgressBar progressBar,boolean prefix) throws FileNotFoundException {
            Reader reader = null;
            if (progressBar  == null)
                  progressBar = new JProgressBar();
            String alphaBet = this.getAlphaBet();
            reader = new BufferedReader(new FileReader(corpusFile));
            String word = new String("empty");
            word = new String(readWord(reader,alphaBet/*"., :;()\"+\n?![]{}*\r\t"*/,progressBar,prefix));
            int i=0;
            while (!(word.equals(new String("")))) {
                  i++;
                  /* repaint the progress bar every 50 words */
                  if (i%50 == 0)
                        progressBar.paint(progressBar.getGraphics());
                  word.toLowerCase();
                  Integer numberOfAppearances = (Integer)wordAppearances.get((Object)word);
                  if (numberOfAppearances == null)
                        numberOfAppearances = new Integer(1);
                  else
                        numberOfAppearances = new Integer((int)numberOfAppearances.intValue() + 1);
                  wordAppearances.put(word,numberOfAppearances);
                  numberOfWords++;
                  word = new String(readWord(reader,alphaBet/*"., :;()\"+\n?![]{}*\r\t"*/,progressBar,prefix));
            }
            try {
                  reader.close();
            }
            catch(IOException exception) {
                  return 0;
            }
            return i;
      }

      public Iterator  getWordIterator() {
            return wordAppearances.keySet().iterator();
      }

      public Set getWords() {
            return wordAppearances.keySet();
      }

      public int length() {
            return numberOfWords;
      }

      public int getWordFrequency(String word) {
            Integer numberOfAppearances = (Integer)wordAppearances.get(word);
            if (numberOfAppearances == null)
                  return 0;
            return numberOfAppearances.intValue();
      }

      public int getStemFrequency(IStem stem) {
            Integer cached = (Integer)stemAppearances.get(stem.toString());
            if (cached != null) {
                  return cached.intValue();
            }
            int stemFrequency = 0;
            Iterator wordIter = getWordIterator();
            while(wordIter.hasNext())
            {
                  String word = (String)wordIter.next();
                  if (word.startsWith(stem.toString())) {
                        Integer numberOfAppearances = (Integer)wordAppearances.get((Object)word);
                        stemFrequency += numberOfAppearances.intValue();
                  }
            }
            stemAppearances.put(stem.toString(),new Integer(stemFrequency));
            return stemFrequency;
      }

      public int getStemFrequency(IStem stem,IMorphology morphology) {
            int result = 0;
            Iterator signIter = morphology.getSignatures().iterator();
            while (signIter.hasNext()) {
                  ISignature signature = (ISignature)signIter.next();
                  if (signature.getStems().contains(stem)) {
                        Iterator suffixIter = signature.getSuffixs().iterator();
                        while (suffixIter.hasNext()) {
                              ISuffix suffix = (ISuffix)suffixIter.next();
                              StringBuffer word = new StringBuffer(stem.toString());
                              if (!(suffix.toString().equals("#"))) {
                                    word.append(suffix.toString());
                              }
                              result += this.getWordFrequency(word.toString());
                        }
                  }
            }
            return result;
      }

      public int getSuffixFrequency(ISuffix suffix) {
            Integer cached = (Integer)suffixAppearances.get(suffix.toString());
            if (cached != null) {
                  return cached.intValue();
            }
            if (suffix.toString().equals("#"))
                  return this.length();
            int suffixFrequency = 0;
            Iterator wordIter = getWordIterator();
            while(wordIter.hasNext())
            {
                  String word = (String)wordIter.next();
                  if (word.endsWith(suffix.toString()) && word.length() > suffix.toString().length()) {
                        Integer numberOfAppearances = (Integer)wordAppearances.get((Object)word);
                        suffixFrequency += numberOfAppearances.intValue();
                  }
            }
            suffixAppearances.put(suffix.toString(),new Integer(suffixFrequency));
            return suffixFrequency;
      }

      public int getSuffixFrequency(ISuffix suffix,IMorphology morphology) {
            int result = 0;
            Iterator signIter = morphology.getSignatures().iterator();
            while (signIter.hasNext()) {
                  ISignature signature = (ISignature)signIter.next();
                  if (signature.getSuffixs().contains(suffix)) {
                        Iterator stemIter = signature.getStems().iterator();
                        while (stemIter.hasNext()) {
                              IStem stem = (IStem)stemIter.next();
                              StringBuffer word = new StringBuffer(stem.toString());
                              if (!(suffix.toString().equals("#"))) {
                                    word.append(suffix.toString());
                              }
                              result += this.getWordFrequency(word.toString());
                        }
                  }
            }
            return result;
      }

      public int getStemInSignatureFrequency(ISignature signature, IStem stem) {
            int frequency = 0;
            Iterator wordIter = signature.getWords().iterator();
            while(wordIter.hasNext()) {
                  String word = (String)wordIter.next();
                  if (word.startsWith(stem.toString())) {
                        Integer numberOfAppearances = (Integer)wordAppearances.get(word);
                        frequency += numberOfAppearances.intValue();
                  }
            }
            return frequency;
      }

      public int getSuffixInSignatureFrequency(ISignature signature, ISuffix suffix) {
            int frequency = 0;
            if (suffix.toString().equals("#")) {
                  Iterator stemIter = signature.getStems().iterator();
                  while (stemIter.hasNext()) {
                        IStem stem = (IStem)stemIter.next();
                        frequency += this.getWordFrequency(stem.toString());
                  }
                  return frequency;
            }
            Iterator wordIter = signature.getWords().iterator();
            while(wordIter.hasNext()) {
                  String word = (String)wordIter.next();
                  if (word.endsWith(suffix.toString()) && !word.equals(suffix.toString())) {
                        Integer numberOfAppearances = (Integer)wordAppearances.get(word);
                        if (numberOfAppearances == null) {
                              System.out.println(word);
                              signature.print();
                        }
                        frequency += numberOfAppearances.intValue();
                  }
            }
            return frequency;
      }

      public int getSignatureFrequency(ISignature signature) {
            int frequency = 0;
            Iterator wordIter = getWordIterator();
            while(wordIter.hasNext()) {
                  String word = (String)wordIter.next();
                  if (signature.getWords().contains(word)) {
                        Integer numberOfAppearances = (Integer)wordAppearances.get(word);
                        frequency += numberOfAppearances.intValue();
                  }
            }
            return frequency;
      }

      public int getLetterFrequency(char letter) {
            Character ch = new Character(letter);
            Integer numberOfAppearances = (Integer)letterAppearances.get(ch);
            if (numberOfAppearances == null)
                  return 0;
            return numberOfAppearances.intValue();
      }

      public int getTotalKgram(int k) {
            Integer cached = (Integer)this.totalKgrams.get(new Integer(k));
            if (cached != null) {
                  return cached.intValue();
            }
            int totalKgrams = 0;
            Iterator wordIter = getWordIterator();
            while(wordIter.hasNext()) {
                  String word = (String)wordIter.next();
                  if (word.length() > k-1) {
                        Integer numberOfAppearances = (Integer)wordAppearances.get(word);
                        totalKgrams += numberOfAppearances.intValue();
                  }
            }
            this.totalKgrams.put(new Integer(k),new Integer(totalKgrams));
            return totalKgrams;
      }

      public int getNumberOfSuccessors(IStem stem) {
            JSet successors = new JSet();
            Iterator wordIter = wordAppearances.keySet().iterator();
            while(wordIter.hasNext()) {
                  String word = (String)wordIter.next();
                  if (word.startsWith(stem.toString())) {
                        if (word.length() == stem.toString().length())
                              successors.add(new Character('#'));
                        else
                              successors.add(new Character(word.charAt(stem.toString().length())));
                  }
            }
            return successors.size();
      }

      private boolean inSet(Set set,Object suffix) {
            Iterator iter = set.iterator();
            while(iter.hasNext()) {
                  Object aSuffix = iter.next();
                  if (aSuffix.toString().compareTo(suffix.toString()) == 0)
                        return true;
            }
            return false;
      }

      public void print() {
            Iterator wordIter = this.getWordIterator();
            while(wordIter.hasNext()) {
                  String word = (String)wordIter.next();
                  System.out.println("Word - " + word);
            }
      }
}