package automorphology.core;

import java.util.List;
import javax.swing.JProgressBar;

/**
 * <p>Title: Auto-Morphology</p>
 * <p>Description: A morphological analyzer based on Goldsmith's algorithm</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Haifa University</p>
 * @author Yaniv Alamaru, Ofer Senderovich
 * @version 1.0
 */

/**
 * IBootstrapHeuristic is an interface which represents a bootstrap
 * heuristic in goldsmith's algorithm. The bootstrap heuristic is a
 * function which takes a word from a corpus and decides where is should
 * be cut into stem and suffix.
 * @author Yaniv Alamaru, Ofer Senderovich
 * <p>Company: Haifa University</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * @version 1.0
 */
public interface IBootstrapHeuristic {

      /**
       * The method analyzes the corpus and creates a morphology
       * accroding to the result of the bootstrap heuristic. The
       * heurisitic can encounter a situation where for a word there
       * could be several good candidates for a cut. In order to resolve
       * it there is a list of other bootstrap heuristics which can be
       * used for the problematic word.
       * @param corpus The corpus to be analyzed.
       * @param progressBar A progress bar which monitors the analysis progress.
       * @return The morphology which represent the analysis result.
       */
      public IMorphology analyze(ICorpus corpus,JProgressBar progressBar);
}