package automorphology.core;

/**
 * <p>Title: Auto-Morphology</p>
 * <p>Description: A morphological analyzer based on Goldsmith's algorithm</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Haifa University</p>
 * @author Yaniv Alamaru, Ofer Senderovich
 * @version 1.0
 */

/**
 * The IStem interface represents a stem in a morphology. The interface
 * defines the functionality which is needed in order to calculate
 * the description length of a stem in a certain morphology of a corpus.
 * @author Yaniv Alamaru, Ofer Senderovich
 * <p>Company: Haifa University</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * @version 1.0
 */
public interface IStem extends Comparable{

      /**
       * The method returns the description length of the stem. The
       * description length of the stem is defined to be the number of
       * bits which represent the stem's letters plus the number of bits
       * needed to represent a pointer to the stem. The size of the pointer
       * depends on the stem's frequency in the corpus.
       * @param corpus The corpus to use in order to find the stem's frequency.
       * @param morphology The morphology which the stem belongs to.
       * @return The stem's description length in bits.
       */
      public double descriptionLength(ICorpus corpus,IMorphology morphology);

      /**
       * The method returns the length of the stem's pointer. The length
       * of the pointer depends on the stem's frequency in the corpus.
       * @param corpus The corpus to use in order to find the stem's frequency.
       * @param morphology The morphology which the stem belongs to.
       * @return The stem's pointer length in bits.
       */
      public double pointerLength(ICorpus corpus,IMorphology morphology);

      /**
       * @return The stem as a string.
       */
      public String toString();

      public boolean equals(Object stem);
      public int compareTo(Object o);
}