package automorphology.core;

/**
 * <p>Title: Auto-Morphology</p>
 * <p>Description: A morphological analyzer based on Goldsmith's algorithm</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Haifa University</p>
 * @author Yaniv Alamaru, Ofer Senderovich
 * @version 1.0
 */

/**
 * The ISuffix interface represents a suffix in a morphology. The
 * interface defines the functionality which is needed in order to
 * calculate the description length of a suffix in a certain
 * morphology of a corpus.
 * @author Yaniv Alamaru, Ofer Senderovich
 * <p>Company: Haifa University</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * @version 1.0
 */
public interface ISuffix extends Comparable{

      /**
       * The method returns the description length of the suffix. The
       * description length of the suffix is defined to be the number of
       * bits which represent the suffix's letters plus the number of bits
       * needed to represent a pointer to the suffix. The size of the
       * pointer depends on the suffix's frequency in the corpus and the
       * total number of instances of analyzed words in a morphology.
       * @param corpus The corpus to use in order to find the suffix's frequency.
       * @param morphology The morphology which defines the set of analyzed words.
       * @return The suffix's description length in bits.
       */
      public double descriptionLength(ICorpus corpus, IMorphology morphology);

      /**
       * The method returns the length of the suffix's pointer. The length
       * of the pointer depends on the suffix's frequency in the corpus
       * and the total number of instances of analyzed words in a morphology.
       * @param corpus The corpus to use in order to find the suffix's frequency.
       * @param morphology The morphology which defines the set of analyzed words.
       * @return The suffix's pointer length in bits.
       */
      public double pointerLength(ICorpus corpus, IMorphology morphology);

      /**
       * @return The suffix as a string.
       */
      public String toString();

      public boolean equals(Object suffix);

      public int compareTo(Object o);
}