package automorphology.core;

import java.util.*;
import javax.swing.JProgressBar;

/**
 * <p>Title: Auto-Morphology</p>
 * <p>Description: A morphological analyzer based on Goldsmith's algorithm</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Haifa University</p>
 * @author Yaniv Alamaru, Ofer Senderovich
 * @version 1.0
 */

public class KgramHeuristic implements IBootstrapHeuristic {

      private int maxKgram = 6;
      private int numberOfSuffix = 100;
      Vector topSuffixs = new Vector();

      class WeightedSuffix implements Comparable {
            private ISuffix suffix;
            private double weight;
            WeightedSuffix(ISuffix suffix,double weight) {
                  this.suffix = suffix;
                  this.weight = weight;
            }
            public int compareTo(Object o) {
                  WeightedSuffix ws = (WeightedSuffix)o;
                  if (ws.weight > this.weight)
                        return -1;
                  if (ws.weight < this.weight)
                        return 1;
                  return 0;
            }
      }

      private void findTopSuffixs(ICorpus corpus,JProgressBar progressBar) {
            Set topSuffixs = new TreeSet();
            Iterator wordIter = corpus.getWordIterator();
            progressBar.setMaximum(corpus.getWords().size());
            progressBar.setValue(0);
            int progressMark = 0;
            while(wordIter.hasNext()) {
                  if ((++progressMark % 1000) == 0) {
                       System.err.print('.');
                       System.err.flush();
                  }
                  String word = (String)wordIter.next();
                  progressBar.setValue(progressBar.getValue()+1);
                  if (progressBar.getValue()%50 == 0)
                        progressBar.paint(progressBar.getGraphics());
                  for(int i=2;((i<=maxKgram) && (i<=word.length()-4));i++){
                        ISuffix suffix = new Suffix(word.substring(word.length()-i+1,word.length()));
                        int suffixFrequency = corpus.getSuffixFrequency(suffix);
                        double weight = suffixFrequency;
                        for(int j=0;j<i-1;j++) {
                              weight /= (double)corpus.getLetterFrequency(suffix.toString().charAt(suffix.toString().length()-j-1));
                        }
                        weight = Math.abs(Math.log(weight)/Math.log(2));
                        weight *= suffixFrequency/(double)corpus.getTotalKgram(i);;
                        WeightedSuffix ws = new WeightedSuffix(suffix,weight);
                        if (topSuffixs.size() < numberOfSuffix) {
                              topSuffixs.add(ws);
                              continue;
                        }
                        topSuffixs.remove(topSuffixs.iterator().next());
                        topSuffixs.add(ws);
                  }
            }
            Iterator it = topSuffixs.iterator();
            //Vector string1 = new Vector(50);
            //Vector string2 = new Vector(50);
            if (topSuffixs.size() > 50) {
                  for(int i=0;i<topSuffixs.size()-50;i++)
                        it.next();
            }
            while(it.hasNext()) {
                  WeightedSuffix ws = (WeightedSuffix)it.next();
                  ISuffix suffix = ws.suffix;
                  /*
                  if (o>=50)
                        string2.add(suffix.toString());
                  else
                        string1.add(suffix.toString());
                  double weight = ws.weight;
                  */
                  this.topSuffixs.add(suffix);
            }
/*
            System.err.println("Top Suffixs");
            for(int i=0;i<50;i++) {
                  System.err.println(i + ")" + this.topSuffixs.elementAt(i).toString());
            }
*/
      }

      public IMorphology analyze(ICorpus corpus,JProgressBar progressBar) {
            int progressMark = 0;
            if (progressBar == null)
                  progressBar = new JProgressBar();
            Map stemToSuffixs = new HashMap();
            Morphology morphology = new Morphology();
            findTopSuffixs(corpus,progressBar);
            int index = 0;
            Iterator wordIter = corpus.getWordIterator();
            while(wordIter.hasNext()) {
                  if ((++progressMark % 1000) == 0) {
                        System.err.print('.');
                        System.err.flush();
                  }
                  boolean wordWasAnalyzed = false;
                  ISuffix longestSuffix = null;
                  String word = (String)wordIter.next();
                  Iterator suffixIter = this.topSuffixs.iterator();
                  while(suffixIter.hasNext()) {
                        ISuffix suffix = (ISuffix)suffixIter.next();
                        /* Stem has to be at least 3 letters long */
                        if (word.endsWith(suffix.toString()) && word.length() > suffix.toString().length()+3) {
                              wordWasAnalyzed = true;
                              if (longestSuffix == null) {
                                    longestSuffix = suffix;
                              }
                              else {
                                    if (longestSuffix.toString().length() < suffix.toString().length())
                                          longestSuffix = suffix;
                              }
                        }
                  }
                  if (wordWasAnalyzed) {
                        IStem stem = new SimpleStem(word.substring(0,word.length()-longestSuffix.toString().length()));
                        morphology.setStemFrequency(stem,morphology.getStemFrequency(stem) + corpus.getWordFrequency(word));
                        morphology.setSuffixFrequency(longestSuffix,morphology.getSuffixFrequency(longestSuffix) + corpus.getWordFrequency(word));
                        JSet suffixs = (JSet)stemToSuffixs.get(stem.toString());
                        if (suffixs == null) {
                              JSet suffixSet = new JSet();
                              suffixSet.add(longestSuffix);
                              stemToSuffixs.put(stem.toString(),suffixSet);
                        }
                        else {
                              suffixs.add(longestSuffix);
                              stemToSuffixs.put(stem.toString(),suffixs);
                        }
                  }
                  if (!wordWasAnalyzed) {
                        IStem stem = new SimpleStem(word);
                        ISuffix nullSuffix = new Suffix("#");
                        morphology.setStemFrequency(stem,morphology.getStemFrequency(stem) + corpus.getWordFrequency(word));
                        morphology.setSuffixFrequency(nullSuffix,morphology.getSuffixFrequency(nullSuffix) + corpus.getWordFrequency(word));
                        JSet suffixSet = (JSet)stemToSuffixs.get(stem.toString());
                        if (suffixSet == null) {
                              suffixSet = new JSet();
                        }
                        suffixSet.add(nullSuffix);
                        stemToSuffixs.put(stem.toString(),suffixSet);
                        //System.err.println(index++ + ") " + stem.toString() + " | " + nullSuffix.toString());
                  }
            }
            //System.err.println("start create");
            //System.err.flush();
            morphology.create(stemToSuffixs);
            //System.err.println("end create");
            //System.err.flush();
            System.err.println();
            System.err.flush();
            return morphology;
      }
}