package automorphology.core;

import java.util.*;
/**
 * <p>Title: Auto-Morphology</p>
 * <p>Description: A morphological analyzer based on Goldsmith's algorithm</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Haifa University</p>
 * @author Yaniv Alamaru, Ofer Senderovich
 * @version 1.0
 */

public class MDL implements IMDL {

      private double calcCompressedCorpusSize(ICorpus corpus,IMorphology morphology) {
            int i=0;
            int progressMark = 0;
            Iterator signIter = morphology.getSignatures().iterator();
            double compressedCorpusSize = 0;
            while(signIter.hasNext()) {
                  ISignature signature = (ISignature)signIter.next();
                  if (((++progressMark % 100) == 0) || (signature.getStems().size() > 200)) {
                        System.err.print('.');
                        System.err.flush();
                  }
                  int signatureFrequency = corpus.getSignatureFrequency(signature);
                  Iterator wordIter = signature.getWords().iterator();
                  while(wordIter.hasNext()) {
                        double compressedWordSize = 0;
                        String word = (String)wordIter.next();
                        int wordFrequency = corpus.getWordFrequency(word);
                        IStem stem = signature.getStem(word);
                        ISuffix suffix = signature.getSuffix(word);
                        if (stem == null) {
                              System.err.println("Failed to find a stem for the word: " + word);
                              System.exit(1);
                        }
                        if (suffix == null) {
                              System.err.println("Failed to find a suffix for the word: " + word);
                              System.exit(1);
                        }
                        compressedWordSize = corpus.length()/(double)corpus.getStemFrequency(stem,morphology);
                        compressedWordSize *= signatureFrequency/(double)corpus.getSuffixInSignatureFrequency(signature,suffix);
                        compressedWordSize = Math.log(compressedWordSize)/Math.log(2);
                        compressedWordSize *= wordFrequency;
                        compressedCorpusSize += compressedWordSize;
                        //System.err.println("Word " + word + " - " + compressedWordSize);
                  }
            }
            System.err.println();
            System.err.flush();
            return compressedCorpusSize;
      }

      public double calc(ICorpus corpus, IMorphology morphology) {
            return morphology.descriptionLength(corpus) + calcCompressedCorpusSize(corpus,morphology);
      }
}