package automorphology.core;

import java.util.*;
import java.io.*;

/**
 * <p>Title: Auto-Morphology</p>
 * <p>Description: A morphological analyzer based on Goldsmith's algorithm</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Haifa University</p>
 * @author Yaniv Alamaru, Ofer Senderovich
 * @version 1.0
 */

public class Morphology implements IMorphology {

      private JSet stems = new JSet();
      private JSet suffixs = new JSet();
      private JSet signatures = new JSet();
      private boolean dirty = false;
      private Integer Wa = null;
      private ISignature nullSignature;
      private HashMap stemsFrequency = new HashMap();
      private HashMap suffixsFrequency = new HashMap();

      public Morphology() {
            JSet suffixs = new JSet();
            suffixs.add(new Suffix("#"));
            ISignature nullSignature = new Signature(suffixs);
            this.signatures.add(nullSignature);
            this.nullSignature = nullSignature;
      }

      public IMorphology getNewMorphology() {
            Morphology mor = new Morphology();
            Iterator mapIter = stemsFrequency.keySet().iterator();
            while(mapIter.hasNext()) {
                  String stem = new String((String)mapIter.next());
                  Integer stemFreq = new Integer(((Integer)stemsFrequency.get(stem)).intValue());
                  mor.stemsFrequency.put(stem,stemFreq);
            }
            mapIter = suffixsFrequency.keySet().iterator();
            while(mapIter.hasNext()) {
                  String suffix = new String((String)mapIter.next());
                  Integer suffixFreq = new Integer(((Integer)suffixsFrequency.get(suffix)).intValue());
                  mor.suffixsFrequency.put(suffix,suffixFreq);
            }
            return mor;
      }

      public ISignature getNullSigature() {
            return nullSignature;
      }

      public void addSignatureSimple(ISignature signature) {
            if (signature.getStems().size() == 0)
                  return;
            if (nullSignature.getSuffixs().equals(signature.getSuffixs())) {
                  nullSignature.getStems().addAll(signature.getStems());
                  stems.addAll(signature.getStems());
                  return;
            }
            signatures.add(signature);
            Iterator suffixIter = signature.getSuffixs().iterator();
            while(suffixIter.hasNext()) {
                  ISuffix suffix = (ISuffix)suffixIter.next();
                  suffixs.add(suffix);
            }
            Iterator stemIter = signature.getStems().iterator();
            while(stemIter.hasNext()) {
                  IStem stem = (IStem)stemIter.next();
                  stems.add(stem);
            }
      }

      public void addSignature(ISignature signature) {
            JSet newSignatures = new JSet();
            Iterator signIter = this.signatures.iterator();
            while(signIter.hasNext()) {
                  if(signature.getStems().size() == 0)
                        return;
                  ISignature currSignature = (ISignature)signIter.next();
                  Iterator stemIter = signature.getStems().iterator();
                  ISignature newSignature = null;
                  while(stemIter.hasNext()) {
                        IStem stem = (IStem)stemIter.next();
                        if (currSignature.getStems().contains(stem)) {
                              stemIter.remove();
                              if (!(currSignature.getSuffixs().equals(signature.getSuffixs()))) {
                                    if (newSignature == null) {
                                          JSet suffixs = new JSet(signature.copy().getSuffixs());
                                          suffixs.addAll(currSignature.getSuffixs());
                                          newSignature = new Signature(suffixs);
                                    }
                                    newSignature.addStem(stem);
                                    currSignature.getStems().remove(stem);
                              }
                        }
                  }
                  if (currSignature.getStems().size() == 0) {
                        signIter.remove();
                        continue;
                  }
                  if (newSignature != null) {
                        newSignatures.add(newSignature);
                        this.suffixs.addAll(newSignature.getSuffixs());
                  }
                  if (currSignature.getSuffixs().equals(signature.getSuffixs())) {
                        this.stems.addAll(signature.getStems());
                        currSignature.getStems().addAll(signature.getStems());
                        this.signatures.addAll(newSignatures);
                        return;
                  }
            }
            this.signatures.addAll(newSignatures);
            if(signature.getStems().size() == 0)
                  return;
            this.signatures.add(signature);
            this.stems.addAll(signature.getStems());
            this.suffixs.addAll(signature.getSuffixs());
      }

      public void addSignature2(ISignature signature) {
            JSet newSignatures = new JSet();
            Iterator signIter = signatures.iterator();
            while (signIter.hasNext()) {
                  ISignature aSignature = (ISignature)signIter.next();
                  JSet suffixs = new JSet(signature.getSuffixs());
                  suffixs.addAll(aSignature.getSuffixs());
                  ISignature newSignature = new Signature(suffixs);
                  Iterator stemIter = aSignature.copy().getStems().iterator();
                  while(stemIter.hasNext()) {
                        IStem stem = (IStem)stemIter.next();
                        if (signature.containsStem(stem)) {
                              newSignature.addStem(new SimpleStem(stem.toString()));
                              aSignature.removeStem(stem);
                              signature.removeStem(stem);
                              if (aSignature.getStems().size() == 0)
                                    signIter.remove();
                        }
                  }
                  if (newSignature.getStems().size() > 0) {
                        newSignatures.add(newSignature);
                  }

                  /* in case there is a signature with the same suffixs */
                  if (signature.getSuffixs().equals(aSignature.getSuffixs())) {
                        ISignature signature2 = aSignature.copy();
                        stemIter = signature.getStems().iterator();
                        while(stemIter.hasNext()) {
                              IStem stem = (IStem)stemIter.next();
                              signature2.addStem(stem);
                              this.stems.add(stem);
                        }
                        signatures.remove(aSignature);
                        if (signature2.getStems().size() > 0)
                              signatures.add(signature2);
                        signatures.addAll(newSignatures);
                        return;
                  }
            }
            if (newSignatures.size() > 0)
                  signatures.addAll(newSignatures);

            /* there is no signature with the same suffixs */
            if (signature.getStems().size() == 0)
                  return;
            signatures.add(signature);
            Iterator suffixIter = signature.getSuffixs().iterator();
            while(suffixIter.hasNext()) {
                  ISuffix suffix = (ISuffix)suffixIter.next();
                  suffixs.add(suffix);
            }
            Iterator stemIter = signature.getStems().iterator();
            while(stemIter.hasNext()) {
                  IStem stem = (IStem)stemIter.next();
                  stems.add(stem);
            }
      }

      public double descriptionLength(ICorpus corpus) {
            double signatureDL = 0;
            /* Suffixs */
            Iterator suffixIter = suffixs.iterator();
            System.err.print('.');
            System.err.flush();

            while(suffixIter.hasNext()) {
                  ISuffix suffix = (ISuffix)suffixIter.next();
                  signatureDL += suffix.descriptionLength(corpus,this);
            }
            signatureDL += Math.log(suffixs.size())/Math.log(2);

            System.err.print('.');
            System.err.flush();

            /* Stems */
            Iterator stemIter = stems.iterator();
            while(stemIter.hasNext()) {
                  IStem stem = (IStem)stemIter.next();
                  signatureDL += stem.descriptionLength(corpus,this);
            }
            signatureDL += Math.log(stems.size())/Math.log(2);

            int progressMark = 0;
            /* Signatures */
            Iterator signatureIter = signatures.iterator();
            while(signatureIter.hasNext()) {
                  ISignature signature = (ISignature)signatureIter.next();
                  if (((++progressMark % 100) == 0) || (signature.getStems().size() > 200)) {
                        System.err.print('.');
                        System.err.flush();
                  }
                  signatureDL += signature.descriptionLength(corpus,this);
            }
            signatureDL += Math.log(signatures.size())/Math.log(2);
            return signatureDL;
      }

      public ISignature getSignature(String word) {
            Iterator signatureIter = signatures.iterator();
            while(signatureIter.hasNext()) {
                  ISignature signature = (ISignature)signatureIter.next();
                  if (signature.getWords().contains(word)) {
                        return signature;
                  }
            }
            return null;
      }

      public int getNumberOfAnalyzedWord(ICorpus corpus) {
            if ((dirty == false) && (this.Wa != null))
                  return this.Wa.intValue();
            int Wa = 0;
            Iterator wordIter = corpus.getWordIterator();
            while(wordIter.hasNext()) {
                  String word = (String)wordIter.next();
                  Iterator signatureIter = signatures.iterator();
                  while(signatureIter.hasNext()) {
                        ISignature signature = (ISignature)signatureIter.next();
                        if ((signature.getSuffixs().size() == 1) && (signature.getSuffixs().contains(new Suffix("#")))) {
                              int Wua = 0;
                              Iterator  signWordsIter = signature.getWords().iterator();
                              while (signWordsIter.hasNext()) {
                                    String signWord = (String)signWordsIter.next();
                                    Wua += corpus.getWordFrequency(signWord);
                              }
                              this.Wa = new Integer(corpus.length() - Wua);
                              dirty = false;
                              return this.Wa.intValue();
                        }
                  }
            }
            this.Wa = new Integer(corpus.length());
            dirty = false;
            return this.Wa.intValue();
      }

      public int getNumberOfSimpleWords(ICorpus corpus) {
            return corpus.length();
      }

      public void print() {
            Iterator signaturesIter = signatures.iterator();
            while(signaturesIter.hasNext()) {
                  ISignature signature = (ISignature)signaturesIter.next();
                  signature.print();
            }
      }

      public JSet getSignatures() {
            return signatures;
      }

      private boolean setsEquality(Set set1,Set set2) {
            if (set1.size() != set2.size())
                  return false;
            Iterator it1 = set1.iterator();
            while (it1.hasNext()) {
                  boolean contained = false;
                  ISuffix suffix1 = (ISuffix)it1.next();
                  Iterator it2 = set2.iterator();
                  while(it2.hasNext()) {
                        ISuffix suffix2 = (ISuffix)it2.next();
                        if (suffix1.equals(suffix2))
                              contained = true;
                  }
                  if (contained == false)
                        return false;
            }
            return true;
      }

      public void create(Map stemToSuffixs) {
            JSet signatures = new JSet();
            int progressMark = 0;
            while(!stemToSuffixs.isEmpty()) {
                  if ((++progressMark % 50) == 0) {
                        System.err.print('.');
                        System.err.flush();
                  }
                  IStem stem = new SimpleStem((String)stemToSuffixs.keySet().iterator().next());
                  JSet suffixs = (JSet)stemToSuffixs.get(stem.toString());
                  ISignature signature = new Signature(suffixs);
                  signature.addStem(stem);
                  stemToSuffixs.remove(stem.toString());
                  //System.err.println("1) Stem - " + stem.toString());
                  Iterator stemIter = stemToSuffixs.keySet().iterator();
                  JSet stems = new JSet(stemToSuffixs.keySet());
                  Iterator stemToSuffixIter = stems.iterator();
                  IStem candidateStem;
                  while(stemToSuffixIter.hasNext()) {
                        candidateStem = new SimpleStem((String)stemToSuffixIter.next());
                        JSet candidateSuffixs = (JSet)stemToSuffixs.get(candidateStem.toString());
                        if (candidateSuffixs.equals(suffixs)) {
                              signature.addStem(candidateStem);
                              stemToSuffixs.remove(candidateStem.toString());
                              //System.err.println("2) Stem - " + candidateStem.toString());
                        }
                  }
                  //this.addSignature(signature);
                  this.addSignatureSimple(signature);
            }
      }

      public void optimize() {
            JSet signatures = new JSet(this.signatures);
            this.signatures.clear();
            this.stems.clear();
            this.suffixs.clear();
            Iterator signIter = signatures.iterator();
            while(signIter.hasNext()) {
                  ISignature signature = (ISignature)signIter.next();
                  this.addSignature(signature);
            }
      }

      public int getNumberOfWords() {
            int sum = 0;
            Iterator signIter = signatures.iterator();
            while (signIter.hasNext()) {
                  ISignature signature = (ISignature)signIter.next();
                  sum += signature.getWords().size();/*signature.getStems().size() * signature.getSuffixs().size()*/;
            }
            return sum;
      }

      public void setStemFrequency(IStem stem,int num) {
            Integer freq = new Integer(num);
            stemsFrequency.put(stem.toString(),freq);
            if (num == 0) {
                  Iterator stemIter = stems.iterator();
                  while (stemIter.hasNext()) {
                        IStem stem2 = (IStem)stemIter.next();
                        if (stem2.equals(stem))
                              stemIter.remove();
                  }
            }

      }

      public int getStemFrequency(IStem stem) {
            Integer freq = (Integer)stemsFrequency.get(stem.toString());
            if (freq == null)
                  return 0;
            return freq.intValue();
      }

      public void setSuffixFrequency(ISuffix suffix,int num) {
            Integer freq = new Integer(num);
            suffixsFrequency.put(suffix.toString(),freq);
            if (num == 0) {
                  suffixs.remove(suffix);
                  /*
                  Iterator suffixIter = suffixs.iterator();
                  while (suffixIter.hasNext()) {
                        ISuffix suffix2 = (ISuffix)suffixIter.next();
                        if (suffix2.toString().equals(suffix.toString())) {
                              System.out.println(suffix2.toString());
                              suffixs.remove(suffix2);
                  */
                              Iterator it = suffixs.iterator();
                              while(it.hasNext()) {
                                    ISuffix s = (ISuffix)it.next();
                              }
                        //}
                  //}
            }
      }

      public int getSuffixFrequency(ISuffix suffix) {
            Integer freq = (Integer)suffixsFrequency.get(suffix.toString());
            if (freq == null)
                  return 0;
            return freq.intValue();
      }

      public void printAnalysis(PrintStream out,boolean prefix) {
            out.println("Signatures");
            out.println("----------");
            Iterator signIter = signatures.iterator();
            while(signIter.hasNext()) {
                  ISignature signature = (ISignature)signIter.next();
                  Iterator suffixIter = signature.getSuffixs().iterator();
                  while(suffixIter.hasNext()) {
                        ISuffix suffix = (ISuffix)suffixIter.next();
                        if (suffixIter.hasNext())
                              if (prefix) {
                                    out.print((new StringBuffer(suffix.toString())).reverse());
                              }
                              else {
                                    out.print(suffix.toString() + ",");
                              }

                        else {
                              if (prefix) {
                                    out.print((new StringBuffer(suffix.toString())).reverse());
                              }
                              else {
                                    out.print(suffix.toString());
                              }

                        }
                  }
                  out.println(" " + signature.getStems().size());
            }

            out.println("Analyzed corpus");
            out.println("---------------");
            signIter = signatures.iterator();
            while(signIter.hasNext()) {
                  ISignature signature = (ISignature)signIter.next();
                  Iterator stemIter = signature.getStems().iterator();
                  while(stemIter.hasNext()) {
                        IStem stem = (IStem)stemIter.next();
                        if (prefix)
                           out.print((new StringBuffer(stem.toString())).reverse().toString() + '\t');
                        else
                           out.print(stem.toString() + '\t');
                        Iterator suffixIter = signature.getSuffixs().iterator();
                        ISuffix suffix = (ISuffix)suffixIter.next();
                        if (((suffix.toString().compareTo("#"))) == 0) {
                          out.print("NULL");
                        }
                        else {
                          if (prefix)
                            out.print((new StringBuffer(suffix.toString())).reverse());
                          else
                            out.print(suffix.toString());
                        }
                        while(suffixIter.hasNext()) {
                          suffix = (ISuffix)suffixIter.next();
                          if ((suffix.toString().compareTo("#")) == 0) {
                            out.print(".NULL");
                          }
                          else {
                            if (prefix)
                              out.print("." + (new StringBuffer(suffix.toString())).reverse());
                            else
                              out.print("." + suffix.toString());
                          }

                        }
                        out.println();
                  }
            }

      }
}