package automorphology.core;

/**
 * <p>Title: Auto-Morphology</p>
 * <p>Description: A morphological analyzer based on Goldsmith's algorithm</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Haifa University</p>
 * @author Yaniv Alamaru, Ofer Senderovich
 * @version 1.0
 */

public class SimpleStem implements IStem {

      private String stemString;
      private Double cachedPointerLength = null;

      public SimpleStem(String stemString) {
            this.stemString = new String(stemString);
      }

      public String toString() {
            return stemString;
      }

      public double descriptionLength(ICorpus corpus, IMorphology morphology) {
            double dl = stemString.length()*Math.log(26)/Math.log(2);
            dl +=  this.pointerLength(corpus,morphology);
            return dl;
      }

      public double pointerLength(ICorpus corpus, IMorphology morphology) {
            if (cachedPointerLength != null)
                  return cachedPointerLength.doubleValue();
            double stemFlag = Math.log(corpus.length()/(double)morphology.getNumberOfSimpleWords(corpus)) / Math.log(2);
            double pl = Math.log(corpus.length()/(double)morphology.getStemFrequency(this)/*corpus.getStemFrequency(this,morphology)*/)/Math.log(2);
            cachedPointerLength = new Double(stemFlag + pl);
            return cachedPointerLength.doubleValue();
      }

      public boolean equals(Object stem) {
            if (stemString.length() != ((IStem)stem).toString().length())
                  return false;
            return (stemString.compareTo(((IStem)stem).toString()) == 0);
      }

      public int compareTo(Object o) {
            IStem stem = (IStem)o;
            return stemString.compareTo(stem.toString());
      }
}