package automorphology.core;

import java.util.*;
import javax.swing.JProgressBar;

/**
 * <p>Title: Auto-Morphology</p>
 * <p>Description: A morphological analyzer based on Goldsmith's algorithm</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Haifa University</p>
 * @author Yaniv Alamaru, Ofer Senderovich
 * @version 1.0
 */

public class SuccessorPeakHeuristic implements IBootstrapHeuristic {
      private int minimumStemSize = 4;
      public IMorphology analyze(ICorpus corpus, JProgressBar progressBar) {
            int progressMark = 0;
            if (progressBar == null)
                  progressBar = new JProgressBar();
            Morphology morphology = new Morphology();
            progressBar.setMaximum(corpus.getWords().size());
            progressBar.setValue(0);
            Map stemToSuffixs = new HashMap();
            Iterator wordIter = corpus.getWordIterator();
            while (wordIter.hasNext()) {
                  if ((++progressMark % 100) == 0) {
                        System.err.print('.');
                        System.err.flush();
                  }
                  progressBar.setValue(progressBar.getValue()+1);
                  if (progressBar.getValue()%50 == 0)
                        progressBar.paint(progressBar.getGraphics());
                  int index = -1;
                  int maxSucc = 0;
                  String word = (String)wordIter.next();
                  if (word.length() < minimumStemSize + 1) {
                        JSet suffixs = (JSet)stemToSuffixs.get(word);
                        if (suffixs == null) {
                              suffixs = new JSet();
                        }
                        ISuffix nullSuffix = new Suffix("#");
                        suffixs.add(nullSuffix);
                        stemToSuffixs.put(word,suffixs);
                        IStem stem = new SimpleStem(word);
                        morphology.setStemFrequency(stem,morphology.getStemFrequency(stem) + corpus.getWordFrequency(word));
                        morphology.setSuffixFrequency(nullSuffix,morphology.getSuffixFrequency(nullSuffix) + corpus.getWordFrequency(word));
                        continue;
                  }
                  for(int i=minimumStemSize;i<=word.length();i++) {
                        IStem stem = new SimpleStem(word.substring(0,i));
                        int succ = corpus.getNumberOfSuccessors(stem);
                        if (maxSucc < succ) {
                              maxSucc = succ;
                              index = i;
                        }
                  }
                  String calcStem = word.substring(0,index);
                  ISuffix calcSuffix = new Suffix(word.substring(index));
                  if (calcSuffix.toString().equals(""))
                        calcSuffix = new Suffix("#");
                  JSet suffixs = (JSet)stemToSuffixs.get(calcStem);
                  if (suffixs == null) {
                        suffixs = new JSet();
                  }
                  suffixs.add(calcSuffix);
                  stemToSuffixs.put(calcStem,suffixs);
                  IStem stem = new SimpleStem(calcStem);
                  morphology.setStemFrequency(stem,morphology.getStemFrequency(stem) + corpus.getWordFrequency(word));
                  morphology.setSuffixFrequency(calcSuffix,morphology.getSuffixFrequency(calcSuffix) + corpus.getWordFrequency(word));
            }
            morphology.create(stemToSuffixs);
            System.err.println();
            System.err.flush();
            return morphology;
      }
}