package automorphology.core;

import java.util.*;
/**
 * <p>Title: Auto-Morphology</p>
 * <p>Description: A morphological analyzer based on Goldsmith's algorithm</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Haifa University</p>
 * @author Yaniv Alamaru, Ofer Senderovich
 * @version 1.0
 */

public class SuffixReducerHeuristic implements IIncrementalHeuristic {

      public IMorphology analyze(ICorpus corpus, IMorphology oldMorphology) {
            int progressMark = 0;
            IMorphology newMorphology = oldMorphology.getNewMorphology();
            Iterator signIter = oldMorphology.getSignatures().iterator();
            while(signIter.hasNext()) {
                  if ((++progressMark % 10) == 0) {
                        System.err.print('.');
                        System.err.flush();
                  }
                  ISignature signature = (ISignature)signIter.next();
                  Iterator suffixIter = signature.getSuffixs().iterator();
                  Character firstLetter = null;
                  boolean foundLetter = false;
                  while(suffixIter.hasNext()) {
                        ISuffix suffix = (ISuffix)suffixIter.next();
                        if(suffix.toString().equals("#")) {
                              foundLetter = false;
                              break;
                        }
                        if (firstLetter == null)
                              firstLetter = new Character(suffix.toString().charAt(0));
                        else if (firstLetter.charValue() != suffix.toString().charAt(0)) {
                              foundLetter = false;
                              break;
                        }
                        else
                              foundLetter = true;
                  }
                  if (!(foundLetter)) {
                        /* Update frequencies */
                        newMorphology.addSignature(signature.copy());
                        continue;
                  }
                  //signature.print();
                  ISignature signature2 = signature.copy();
                  signature2.borrowLetter(firstLetter.charValue());
                  updateFrequencies(corpus,newMorphology,signature,signature2);
                  newMorphology.addSignature(signature2);
            }
            System.err.println();
            System.err.flush();
            return newMorphology;
      }

      private void updateFrequencies(ICorpus corpus,IMorphology morphology,ISignature oldSignature,ISignature newSignature) {
            Iterator suffixIter = oldSignature.getSuffixs().iterator();
            while (suffixIter.hasNext()) {
                  ISuffix suffix = (ISuffix)suffixIter.next();
                  morphology.setSuffixFrequency(suffix,morphology.getSuffixFrequency(suffix) - corpus.getSuffixInSignatureFrequency(oldSignature,suffix));
            }
            suffixIter = newSignature.getSuffixs().iterator();
            while(suffixIter.hasNext()) {
                  ISuffix suffix = (ISuffix)suffixIter.next();
                  morphology.setSuffixFrequency(suffix,morphology.getSuffixFrequency(suffix) + corpus.getSuffixInSignatureFrequency(newSignature,suffix));
            }
            Iterator stemIter1 = oldSignature.getStems().iterator();
            Iterator stemIter2 = newSignature.getStems().iterator();
            while (stemIter1.hasNext()) {
                  IStem oldStem = (IStem)stemIter1.next();
                  IStem newStem = (IStem)stemIter2.next();
                  morphology.setStemFrequency(newStem,morphology.getStemFrequency(newStem) + corpus.getStemInSignatureFrequency(newSignature,newStem));
                  morphology.setStemFrequency(oldStem,morphology.getStemFrequency(oldStem) - corpus.getStemInSignatureFrequency(oldSignature,oldStem));
            }
      }
}