package automorphology.gui;

import java.io.*;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import automorphology.core.*;
import java.util.*;
/**
 * <p>Title: Auto-Morphology</p>
 * <p>Description: A morphological analyzer based on Goldsmith's algorithm</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Haifa University</p>
 * @author Yaniv Alamaru, Ofer Senderovich
 * @version 1.0
 */

/**
 * AutoMorphologyMainFrame class is a frame which enables the user to
 * load the corpus file and activate the morphological analyzer on the
 * algorithm.
 * @author Yaniv Alamaru, Ofer Senderovich
 * <p>Company: Haifa University</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * @version 1.0
 */
public class AutoMorphologyMainFrame extends JFrame {
      private JMenuBar appMenuBar = new JMenuBar();
      private JMenu fileMenuItem = new JMenu();
      private JMenuItem fileOpenMenuItem = new JMenuItem();
      private JMenu helpMenuItem = new JMenu();
      private JMenuItem helpAboutMenuItem = new JMenuItem();
      private JMenuItem fileExitMenuItem = new JMenuItem();
      private BorderLayout borderLayout1 = new BorderLayout();
      private JFileChooser jFileChooser = new JFileChooser();
      private JPanel jPanel1 = new JPanel();
      private JProgressBar jProgressBar1 = new JProgressBar();
      private JLabel statusBar = new JLabel();
      private BorderLayout borderLayout2 = new BorderLayout();

      public AutoMorphologyMainFrame() {
            try {
                  jbInit();
            }
            catch(Exception e) {
                  e.printStackTrace();
            }
      }
      public static void main(String[] args) {
            try {
                  UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
            }
            catch(Exception e) {
                  e.printStackTrace();
            }
            AutoMorphologyMainFrame autoMorphologyMainFrame = new AutoMorphologyMainFrame();
            //Validate frames that have preset sizes
            //Pack frames that have useful preferred size info, e.g. from their layout
            autoMorphologyMainFrame.validate();
            //Center the window
            Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
            Dimension frameSize = autoMorphologyMainFrame.getSize();
            frameSize.setSize((int)(screenSize.getWidth()*3/4),(int)(screenSize.getHeight()*3/4));
            autoMorphologyMainFrame.setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);
            autoMorphologyMainFrame.setSize(frameSize);
            autoMorphologyMainFrame.setVisible(true);
      }
      private void jbInit() throws Exception {
            fileMenuItem.setText("File");
            fileOpenMenuItem.setText("Open");
            fileOpenMenuItem.addActionListener(new AutoMorphologyMainFrame_fileOpenMenuItem_actionAdapter(this));
            helpMenuItem.setText("Help");
            helpAboutMenuItem.setText("About");
            helpAboutMenuItem.addActionListener(new AutoMorphologyMainFrame_helpAboutMenuItem_actionAdapter(this));
            fileExitMenuItem.setText("Exit");
            fileExitMenuItem.addActionListener(new AutoMorphologyMainFrame_fileExitMenuItem_actionAdapter(this));
            this.setDefaultCloseOperation(EXIT_ON_CLOSE);
            this.setForeground(Color.darkGray);
            this.setJMenuBar(appMenuBar);
            this.setTitle("AutoMorphology");
            this.getContentPane().setLayout(borderLayout1);
            jFileChooser.setDialogTitle("Choose the corpus file");
            jProgressBar1.setForeground(Color.darkGray);
            jProgressBar1.setDoubleBuffered(true);
            jProgressBar1.setStringPainted(true);
            statusBar.setBorder(BorderFactory.createEtchedBorder());
            statusBar.setDoubleBuffered(true);
            statusBar.setOpaque(true);
            statusBar.setDisplayedMnemonic('0');
            statusBar.setText("Idle");
            jPanel1.setLayout(borderLayout2);
            jPanel1.setDoubleBuffered(false);
            appMenuBar.add(fileMenuItem);
            appMenuBar.add(helpMenuItem);
            fileMenuItem.add(fileOpenMenuItem);
            fileMenuItem.addSeparator();
            fileMenuItem.add(fileExitMenuItem);
            helpMenuItem.add(helpAboutMenuItem);
            this.getContentPane().add(jPanel1,  BorderLayout.SOUTH);
            jPanel1.add(jProgressBar1,  BorderLayout.EAST);
            jPanel1.add(statusBar,  BorderLayout.CENTER);
      }

      void fileOpenMenuItem_actionPerformed(ActionEvent e) {
            jFileChooser.setDialogTitle("Choose the alphabet file");
            jFileChooser.setCurrentDirectory(new File("d:\\jbuilder7\\projects\\automorphology"));
            if (JFileChooser.APPROVE_OPTION == jFileChooser.showOpenDialog(this)) {
                  String selectedAlhpaBetFile = new String(jFileChooser.getSelectedFile().getPath());
                  jFileChooser.setDialogTitle("Choose the corpus file");
                  if (jFileChooser.APPROVE_OPTION == jFileChooser.showOpenDialog(this)){
                        String selectedCorpusFile = jFileChooser.getSelectedFile().getPath();
                        Corpus corpus = new Corpus(selectedCorpusFile,selectedAlhpaBetFile);
                        jProgressBar1.setValue(0);
                        if (jFileChooser.getSelectedFile().exists()) {
                              jProgressBar1.setMaximum((int)jFileChooser.getSelectedFile().length());
                              statusBar.setText("Reading " + selectedCorpusFile);
                              this.setTitle("Auto-Morphlogy - " + selectedCorpusFile);
                              this.paint(this.getGraphics());
                              try {
                                    corpus.init(jProgressBar1,false);
                                    //corpus.print();
                                    //return;
                              }
                              catch (Exception exception) {
                                    exception.printStackTrace();
                              }
                        }
                        else {
                              this.setTitle("Auto-Morphology");
                              statusBar.setText("File Not Found!");
                              jProgressBar1.setMaximum((int)jFileChooser.getSelectedFile().length());
                              return;
                        }
                        this.setTitle("Auto-Morphlogy - " + jFileChooser.getSelectedFile().getPath());
                        this.statusBar.setText("Creating morphology using bootstrap heuristic");
                        this.statusBar.update(this.statusBar.getGraphics());

/*
                        JSet set1 = new JSet();
                        JSet set2 = new JSet();
                        IStem stem1 = new SimpleStem("yaniv");
                        IStem stem2 = new SimpleStem("yaniv");
                        set1.add(stem1);
                        set2.add(stem2);
                        set1.equals(set2);
*/
                        IMorphology morphology1 = (new KgramHeuristic()).analyze(corpus,jProgressBar1);
                        //IMorphology morphology1 = (new SuccessorPeakHeuristic()).analyze(corpus,jProgressBar1);
                        IMorphology morphology2 = (new CleanupHeuristic(2,2)).analyze(corpus,morphology1);
                        morphology2.optimize();
                        IMorphology morphology3 = (new SuffixReducerHeuristic()).analyze(corpus,morphology2);
                        morphology3.optimize();
                        morphology1.printAnalysis(System.out,false);
                        IMDL mdl1 = new MDL();
                        IMDL mdl2 = new MDL();
                        IMDL mdl3 = new MDL();
                        System.out.println("1) " + mdl1.calc(corpus,morphology1));
                        System.out.flush();
                        System.out.println("2) " + mdl2.calc(corpus,morphology2));
                        System.out.flush();
                        System.out.println("3) " + mdl3.calc(corpus,morphology3));
                        System.out.flush();
                        morphology3.printAnalysis(System.out,false);
                  }
            }
      }

      void fileExitMenuItem_actionPerformed(ActionEvent e) {
            System.exit(0);
      }

      void helpAboutMenuItem_actionPerformed(ActionEvent e) {
            AutoMorphologyAboutBox dlg = new AutoMorphologyAboutBox(this);
            Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
            Dimension frameSize = dlg.getSize();
            frameSize.setSize((int)(screenSize.getWidth()*3/5),(int)(screenSize.getHeight()*3/5));
            dlg.setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);
            dlg.setSize(frameSize);
            dlg.setVisible(true);
      }
}

class AutoMorphologyMainFrame_fileOpenMenuItem_actionAdapter implements java.awt.event.ActionListener {
      private AutoMorphologyMainFrame adaptee;

      AutoMorphologyMainFrame_fileOpenMenuItem_actionAdapter(AutoMorphologyMainFrame adaptee) {
            this.adaptee = adaptee;
      }
      public void actionPerformed(ActionEvent e) {
            adaptee.fileOpenMenuItem_actionPerformed(e);
      }
}

class AutoMorphologyMainFrame_fileExitMenuItem_actionAdapter implements java.awt.event.ActionListener {
      private AutoMorphologyMainFrame adaptee;

      AutoMorphologyMainFrame_fileExitMenuItem_actionAdapter(AutoMorphologyMainFrame adaptee) {
            this.adaptee = adaptee;
      }
      public void actionPerformed(ActionEvent e) {
            adaptee.fileExitMenuItem_actionPerformed(e);
      }
}

class AutoMorphologyMainFrame_helpAboutMenuItem_actionAdapter implements java.awt.event.ActionListener {
      private AutoMorphologyMainFrame adaptee;

      AutoMorphologyMainFrame_helpAboutMenuItem_actionAdapter(AutoMorphologyMainFrame adaptee) {
            this.adaptee = adaptee;
      }
      public void actionPerformed(ActionEvent e) {
            adaptee.helpAboutMenuItem_actionPerformed(e);
      }
}